Package: rpact
Title: Confirmatory Adaptive Clinical Trial Design and Analysis
Version: 4.4.0
Date: 2026-03-04
Authors@R: c(
	person(
		given = "Gernot", 
		family = "Wassmer", 
		email = "gernot.wassmer@rpact.com", 
		comment = c(ORCID = "0000-0001-9397-1794"), 
		role = c("aut")
	),
    person(
    	given = "Friedrich", 
    	family = "Pahlke", 
    	email = "friedrich.pahlke@rpact.com",
    	comment = c(ORCID = "0000-0003-2105-2582"), 
    	role = c("aut", "cre")
    ),
    person(
    	given = "Till", 
    	family = "Jensen", 
    	email = "till.jensen@rpact.com",
    	role = c("ctb")
	),
    person(
    	given = "Daniel", 
    	family = "Sabanes Bove", 
    	email = "daniel.sabanes_bove@rconis.com",
        comment = c(ORCID = "0000-0002-0176-9239"), 
    	role = c("ctb")
	),
    person(
    	given = "Stephen", 
    	family = "Schueuerhuis", 
    	email = "stephen.schueuerhuis@charite.de",
        comment = c(ORCID = "0009-0006-3124-6056"), 
    	role = c("ctb")
	),
    person(
    	given = "Tobias", 
    	family = "Muetze", 
    	email = "tobias.muetze@novartis.com",
    	comment = c(ORCID = "0000-0002-4111-1941"), 
    	role = c("ctb")
	),
    person(
        "RPACT GmbH", 
        role = c("cph", "fnd"), 
        email = "info@rpact.com"
    ))
Description: Design and analysis of confirmatory adaptive clinical trials with continuous, binary, and survival endpoints according to the methods described in the monograph by Wassmer and Brannath (2025) <doi:10.1007/978-3-031-89669-9>. This includes classical group sequential as well as multi-stage adaptive hypotheses tests that are based on the combination testing principle.
License: LGPL-3
Encoding: UTF-8
LazyData: true
URL: https://www.rpact.org, https://www.rpact.com,
        https://docs.rpact.org, https://github.com/rpact-com/rpact,
        https://rpact-cloud.share.connect.posit.cloud
BugReports: https://github.com/rpact-com/rpact/issues
Language: en-US
Depends: R (>= 3.6.0)
Imports: methods, stats, utils, graphics, tools, rlang, R6 (>= 2.5.1),
        knitr (>= 1.19), Rcpp (>= 1.0.3)
LinkingTo: Rcpp
Suggests: ggplot2 (>= 3.5.0), testthat (>= 3.0.0), rmarkdown (>= 1.10),
        rappdirs (>= 0.3.3)
VignetteBuilder: knitr, rmarkdown
RoxygenNote: 7.3.3
Config/testthat/edition: 3
Config/testthat/parallel: true
Config/testthat/start-first: *analysis*
Collate: 'RcppExports.R' 'f_logger.R' 'class_dictionary.R'
        'f_core_constants.R' 'f_core_utilities.R' 'f_core_assertions.R'
        'f_analysis_utilities.R' 'f_parameter_set_utilities.R'
        'class_core_parameter_set.R' 'class_core_plot_settings.R'
        'f_core_plot.R' 'class_design.R' 'f_object_r_code.R'
        'f_analysis_base.R' 'class_analysis_dataset.R'
        'class_analysis_stage_results.R' 'class_analysis_results.R'
        'f_design_general_utilities.R' 'class_time.R'
        'class_design_set.R' 'class_design_plan.R'
        'class_design_power_and_asn.R' 'class_event_probabilities.R'
        'f_simulation_base_counts.R' 'f_simulation_utilities.R'
        'f_simulation_base_survival.R' 'class_simulation_results.R'
        'class_performance_score.R' 'class_summary.R' 'data.R'
        'f_analysis_base_means.R' 'f_analysis_base_rates.R'
        'f_analysis_base_survival.R'
        'f_analysis_boundary_recalculation.R' 'f_analysis_enrichment.R'
        'f_analysis_enrichment_means.R' 'f_analysis_enrichment_rates.R'
        'f_analysis_enrichment_survival.R' 'f_analysis_multiarm.R'
        'f_analysis_multiarm_means.R' 'f_analysis_multiarm_rates.R'
        'f_analysis_multiarm_survival.R' 'f_as251.R'
        'f_core_output_formats.R' 'f_design_fisher_combination_test.R'
        'f_design_futility_bounds.R' 'f_design_group_sequential.R'
        'f_design_plan_counts.R' 'f_design_plan_means.R'
        'f_design_plan_plot.R' 'f_design_plan_rates.R'
        'f_design_plan_survival.R' 'f_design_plan_utilities.R'
        'f_io_utilities.R' 'f_quality_assurance.R'
        'f_simulation_base_means.R' 'f_simulation_base_rates.R'
        'f_simulation_calc_subjects_function.R'
        'f_simulation_enrichment.R' 'f_simulation_enrichment_means.R'
        'f_simulation_enrichment_rates.R'
        'f_simulation_enrichment_survival.R' 'f_simulation_multiarm.R'
        'f_simulation_multiarm_means.R' 'f_simulation_multiarm_rates.R'
        'f_simulation_multiarm_survival.R'
        'f_simulation_performance_score.R' 'f_simulation_plot.R'
        'parameter_descriptions.R' 'pkgname.R'
NeedsCompilation: yes
Packaged: 2026-03-04 10:01:59 UTC; fried
Author: Gernot Wassmer [aut] (ORCID: <https://orcid.org/0000-0001-9397-1794>),
  Friedrich Pahlke [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-2105-2582>),
  Till Jensen [ctb],
  Daniel Sabanes Bove [ctb] (ORCID:
    <https://orcid.org/0000-0002-0176-9239>),
  Stephen Schueuerhuis [ctb] (ORCID:
    <https://orcid.org/0009-0006-3124-6056>),
  Tobias Muetze [ctb] (ORCID: <https://orcid.org/0000-0002-4111-1941>),
  RPACT GmbH [cph, fnd]
Maintainer: Friedrich Pahlke <friedrich.pahlke@rpact.com>
Repository: CRAN
Date/Publication: 2026-03-04 17:00:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-27 15:55:54 UTC; unix
Archs: rpact.so.dSYM
