# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,AnalysisResults)
S3method(as.data.frame,ParameterSet)
S3method(as.data.frame,PowerAndAverageSampleNumberResult)
S3method(as.data.frame,StageResults)
S3method(as.data.frame,TrialDesign)
S3method(as.data.frame,TrialDesignCharacteristics)
S3method(as.data.frame,TrialDesignPlan)
S3method(as.data.frame,TrialDesignSet)
S3method(as.list,Dictionary)
S3method(as.matrix,FieldSet)
S3method(as.vector,Dictionary)
S3method(fetch,ParameterSet)
S3method(kable,FieldSet)
S3method(kable,ParameterSet)
S3method(kable,array)
S3method(kable,character)
S3method(kable,data.frame)
S3method(kable,logical)
S3method(kable,matrix)
S3method(kable,numeric)
S3method(kable,table)
S3method(knit_print,FieldSet)
S3method(knit_print,ParameterSet)
S3method(knit_print,SummaryFactory)
S3method(length,TrialDesignSet)
S3method(names,AnalysisResults)
S3method(names,FieldSet)
S3method(names,SimulationResults)
S3method(names,StageResults)
S3method(names,TrialDesignSet)
S3method(obtain,ParameterSet)
S3method(plot,AnalysisResults)
S3method(plot,Dataset)
S3method(plot,EventProbabilities)
S3method(plot,NumberOfSubjects)
S3method(plot,ParameterSet)
S3method(plot,SimulationResults)
S3method(plot,StageResults)
S3method(plot,SummaryFactory)
S3method(plot,TrialDesign)
S3method(plot,TrialDesignCharacteristics)
S3method(plot,TrialDesignPlan)
S3method(plot,TrialDesignSet)
S3method(plot,TrialDesignSummaries)
S3method(print,Dataset)
S3method(print,Dictionary)
S3method(print,FieldSet)
S3method(print,FutilityBounds)
S3method(print,InstallationQualificationResult)
S3method(print,ParameterSet)
S3method(print,SummaryFactory)
S3method(print,TrialDesignCharacteristics)
S3method(print,TrialDesignSummaries)
S3method(summary,AnalysisResults)
S3method(summary,Dataset)
S3method(summary,FutilityBounds)
S3method(summary,ParameterSet)
S3method(summary,TrialDesignSet)
export(MarkdownReporter)
export(as251Normal)
export(as251StudentT)
export(checkInstallationQualificationStatus)
export(disableStartupMessages)
export(enableStartupMessages)
export(fetch)
export(getAccrualTime)
export(getAnalysisResults)
export(getAvailablePlotTypes)
export(getClosedCombinationTestResults)
export(getClosedConditionalDunnettTestResults)
export(getConditionalPower)
export(getConditionalRejectionProbabilities)
export(getData)
export(getData.SimulationResults)
export(getDataSet)
export(getDataset)
export(getDesignCharacteristics)
export(getDesignConditionalDunnett)
export(getDesignFisher)
export(getDesignGroupSequential)
export(getDesignInverseNormal)
export(getDesignSet)
export(getEventProbabilities)
export(getFinalConfidenceInterval)
export(getFinalPValue)
export(getFutilityBounds)
export(getGroupSequentialProbabilities)
export(getHazardRatioByLambda)
export(getHazardRatioByMedian)
export(getHazardRatioByPi)
export(getLambda1ByLambda2AndHazardRatio)
export(getLambda2ByLambda1AndHazardRatio)
export(getLambdaByMedian)
export(getLambdaByPi)
export(getLambdaStepFunction)
export(getLogLevel)
export(getLongFormat)
export(getMedianByLambda)
export(getMedianByPi)
export(getNumberOfSubjects)
export(getObjectRCode)
export(getObservedInformationRates)
export(getOutputFormat)
export(getParameterCaption)
export(getParameterName)
export(getParameterType)
export(getPerformanceScore)
export(getPi1ByPi2AndHazardRatio)
export(getPi2ByPi1AndHazardRatio)
export(getPiByLambda)
export(getPiByMedian)
export(getPiecewiseExponentialDistribution)
export(getPiecewiseExponentialQuantile)
export(getPiecewiseExponentialRandomNumbers)
export(getPiecewiseSurvivalTime)
export(getPlotSettings)
export(getPowerAndAverageSampleNumber)
export(getPowerCounts)
export(getPowerMeans)
export(getPowerRates)
export(getPowerSurvival)
export(getRawData)
export(getRepeatedConfidenceIntervals)
export(getRepeatedPValues)
export(getSampleSizeCounts)
export(getSampleSizeMeans)
export(getSampleSizeRates)
export(getSampleSizeSurvival)
export(getSimulationCounts)
export(getSimulationEnrichmentMeans)
export(getSimulationEnrichmentRates)
export(getSimulationEnrichmentSurvival)
export(getSimulationMeans)
export(getSimulationMultiArmMeans)
export(getSimulationMultiArmRates)
export(getSimulationMultiArmSurvival)
export(getSimulationRates)
export(getSimulationSurvival)
export(getStageResults)
export(getSystemIdentifier)
export(getTestActions)
export(getTestLabel)
export(getWideFormat)
export(kable)
export(mvnprd)
export(mvstud)
export(obtain)
export(plotTypes)
export(ppwexp)
export(printCitation)
export(qpwexp)
export(rcmd)
export(readDataset)
export(readDatasets)
export(readKeyValueFile)
export(resetLogLevel)
export(resetOptions)
export(rpwexp)
export(saveOptions)
export(setLogLevel)
export(setOutputFormat)
export(setupPackageTests)
export(testPackage)
export(test_plan_section)
export(writeDataset)
export(writeDatasets)
export(writeKeyValueFile)
import(graphics)
import(methods)
import(stats)
import(tools)
import(utils)
importFrom(Rcpp,evalCpp)
importFrom(knitr,kable)
importFrom(knitr,knit_print)
importFrom(methods,new)
importFrom(rlang,.data)
useDynLib(rpact,.registration = TRUE)
