% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_alpha}
\alias{param_alpha}
\title{Parameter Description: Alpha}
\arguments{
\item{alpha}{The significance level alpha, default is \code{0.025}. Must be a positive numeric of length 1.}
}
\description{
Parameter Description: Alpha
}
\keyword{internal}
