% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_lambda1}
\alias{param_lambda1}
\title{Parameter Description: Lambda (1)}
\arguments{
\item{lambda1}{The assumed hazard rate in the treatment group, there is no default.
\code{lambda1} can also be used to define piecewise exponentially distributed survival times (see details). Must be a positive numeric of length 1.}
}
\description{
Parameter Description: Lambda (1)
}
\keyword{internal}
