% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_median2}
\alias{param_median2}
\title{Parameter Description: Median (2)}
\arguments{
\item{median2}{The assumed median survival time in the reference group, there is no default. Must be a positive numeric of length 1.}
}
\description{
Parameter Description: Median (2)
}
\keyword{internal}
