% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/f_logger.R
\name{resetLogLevel}
\alias{resetLogLevel}
\title{Reset Log Level}
\usage{
resetLogLevel()
}
\description{
Resets the \code{rpact} log level.
}
\details{
This function resets the log level of the \code{rpact} internal log message
system to the default value \code{"PROGRESS"}.
}
\examples{
\dontrun{
# reset log level to default value
resetLogLevel()
}

}
\seealso{
\itemize{
\item \code{\link[=getLogLevel]{getLogLevel()}} for getting the current log level,
\item \code{\link[=setLogLevel]{setLogLevel()}} for setting the log level.
}
}
\keyword{internal}
