% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_design}
\alias{param_design}
\title{Parameter Description: Design}
\arguments{
\item{design}{The trial design.}
}
\description{
Parameter Description: Design
}
\keyword{internal}
