% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_groups}
\alias{param_groups}
\title{Parameter Description: Number Of Treatment Groups}
\arguments{
\item{groups}{The number of treatment groups (1 or 2), default is \code{2}.}
}
\description{
Parameter Description: Number Of Treatment Groups
}
\keyword{internal}
