% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class_summary.R
\name{plot.SummaryFactory}
\alias{plot.SummaryFactory}
\title{Summary Factory Plotting}
\usage{
\method{plot}{SummaryFactory}(x, y, ..., showSummary = FALSE)
}
\arguments{
\item{x}{The summary factory object.}

\item{y}{Not available for this kind of plot (is only defined
to be compatible to the generic plot function).}

\item{...}{Optional plot arguments. At the moment \code{xlim} and \code{ylim} are implemented
for changing x or y axis limits without dropping data observations.}

\item{showSummary}{Show the summary before creating the
plot output, default is \code{FALSE}.}
}
\value{
Returns a \code{ggplot2} object.
}
\description{
Plots a summary factory.
}
\details{
Generic function to plot all kinds of summary factories.
}
