% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class_design_set.R
\name{length.TrialDesignSet}
\alias{length.TrialDesignSet}
\title{Length of Trial Design Set}
\usage{
\method{length}{TrialDesignSet}(x)
}
\arguments{
\item{x}{A \code{\link{TrialDesignSet}} object.}
}
\value{
Returns a non-negative \code{\link[base]{integer}} of length 1
representing the number of design in the \code{TrialDesignSet}.
}
\description{
Returns the number of designs in a \code{TrialDesignSet}.
}
\details{
Is helpful for iteration over all designs in a design set.
}
\examples{
\dontrun{
designSet <- getDesignSet(design = getDesignGroupSequential(), alpha = c(0.01, 0.05))
length(designSet)
}

}
\keyword{internal}
