% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/f_quality_assurance.R
\name{setupPackageTests}
\alias{setupPackageTests}
\title{Setup Package Tests}
\usage{
setupPackageTests(token, secret)
}
\arguments{
\item{token}{A character string representing the token for authentication.}

\item{secret}{A character string representing the secret for authentication.}
}
\value{
The function returns \code{TRUE} if all test files are downloaded and copied successfully to the rpact installation directory; otherwise, it returns \code{FALSE}.
}
\description{
This function sets up the package tests by downloading the test files and copying them to the rpact installation directory.
}
\details{
The function first checks if the \code{rpact} package directory and its \code{tests} and \code{testthat} subdirectories exist.
If they do not exist, it stops with an error. It then downloads the test files to a temporary directory and copies them
to the \code{tests} directory of the \code{rpact} package. If all test files are copied successfully, it removes the default test file.
}
\references{
For more information, please visit: \url{https://www.rpact.org/vignettes/utilities/rpact_installation_qualification/}
}
\keyword{internal}
