% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_intersectionTest_MultiArm}
\alias{param_intersectionTest_MultiArm}
\title{Parameter Description: Intersection Test}
\arguments{
\item{intersectionTest}{Defines the multiple test for the intersection
hypotheses in the closed system of hypotheses.
Five options are available in multi-arm designs: \code{"Dunnett"}, \code{"Bonferroni"}, \code{"Simes"},
\code{"Sidak"}, and \code{"Hierarchical"}, default is \code{"Dunnett"}.}
}
\description{
Parameter Description: Intersection Test
}
\keyword{internal}
