% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_adaptations}
\alias{param_adaptations}
\title{Parameter Description: Adaptations}
\arguments{
\item{adaptations}{A logical vector of length \code{kMax - 1} indicating whether or not an adaptation takes
place at interim k, default is \code{rep(TRUE, kMax - 1)}.}
}
\description{
Parameter Description: Adaptations
}
\keyword{internal}
