% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class_core_parameter_set.R
\name{FieldSet}
\alias{FieldSet}
\title{Field Set}
\description{
Basic class for field sets.
}
\details{
The field set implements basic functions for a set of fields.
}
\keyword{internal}
