% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/f_core_utilities.R
\name{getTestLabel}
\alias{getTestLabel}
\title{Get Test Label}
\usage{
getTestLabel(x)
}
\arguments{
\item{x}{The value to be converted into a test label.}
}
\value{
A character string representing the input value.
}
\description{
Returns a string representation of the input value for use as a test label.
Handles various types, including NULL, NA, vectors, and custom objects.
}
\examples{
\dontrun{
getTestLabel(NULL)
getTestLabel(NA)
getTestLabel(1:3)
getTestLabel(6)
getTestLabel(getDesignFisher())
}

}
\keyword{internal}
