% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_dropoutTime}
\alias{param_dropoutTime}
\title{Parameter Description: Dropout Time}
\arguments{
\item{dropoutTime}{The assumed time for drop-out rates in the control and the
treatment group, default is \code{12}.}
}
\description{
Parameter Description: Dropout Time
}
\keyword{internal}
