% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_lambda2_counts}
\alias{param_lambda2_counts}
\title{Parameter Description: lambda (2) for Counts}
\arguments{
\item{lambda2}{A numeric value that represents the assumed rate of a homogeneous Poisson process in
the control group, there is no default.}
}
\description{
Parameter Description: lambda (2) for Counts
}
\keyword{internal}
