% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_typeOfDesign}
\alias{param_typeOfDesign}
\title{Parameter Description: Type of Design}
\arguments{
\item{typeOfDesign}{The type of design. Type of design is one of the following:
O'Brien & Fleming (\code{"OF"}), Pocock (\code{"P"}), Wang & Tsiatis Delta class (\code{"WT"}),
Pampallona & Tsiatis (\code{"PT"}), Haybittle & Peto ("HP"),
Optimum design within Wang & Tsiatis class (\code{"WToptimum"}),
O'Brien & Fleming type alpha spending (\code{"asOF"}), Pocock type alpha spending (\code{"asP"}),
Kim & DeMets alpha spending (\code{"asKD"}), Hwang, Shi & DeCani alpha spending (\code{"asHSD"}),
user defined alpha spending (\code{"asUser"}), no early efficacy stop (\code{"noEarlyEfficacy"}),
default is \code{"OF"}.}
}
\description{
Parameter Description: Type of Design
}
\keyword{internal}
