% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_effectMatrix}
\alias{param_effectMatrix}
\title{Parameter Description: Effect Matrix}
\arguments{
\item{effectMatrix}{Matrix of effect sizes with \code{activeArms} columns and number of rows
reflecting the different situations to consider.}
}
\description{
Parameter Description: Effect Matrix
}
\keyword{internal}
