% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_accrualTime}
\alias{param_accrualTime}
\title{Parameter Description: Accrual Time}
\arguments{
\item{accrualTime}{The assumed accrual time intervals for the study, default is
\code{c(0, 12)} (for details see \code{\link[=getAccrualTime]{getAccrualTime()}}).}
}
\description{
Parameter Description: Accrual Time
}
\keyword{internal}
