% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class_analysis_dataset.R
\name{Dataset}
\alias{Dataset}
\title{Dataset}
\description{
Basic class for datasets.
}
\details{
\code{Dataset} is the basic class for
\itemize{
\item \code{\link{DatasetMeans}},
\item \code{\link{DatasetRates}},
\item \code{\link{DatasetSurvival}}, and
\item \code{\link{DatasetEnrichmentSurvival}}.
}
This basic class contains the fields \code{stages} and \code{groups} and several commonly used
functions.
}
\section{Fields}{

\describe{
\item{\code{stages}}{The stage numbers of the trial. Is a numeric vector of length \code{kMax} containing whole numbers.}

\item{\code{groups}}{The group numbers. Is a numeric vector.}
}}

\keyword{internal}
