% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_stage}
\alias{param_stage}
\title{Parameter Description: Stage}
\arguments{
\item{stage}{The stage number (optional). Default: total number of existing stages in the data input.}
}
\description{
Parameter Description: Stage
}
\keyword{internal}
