% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_median1}
\alias{param_median1}
\title{Parameter Description: Median (1)}
\arguments{
\item{median1}{The assumed median survival time in the treatment group, there is no default.}
}
\description{
Parameter Description: Median (1)
}
\keyword{internal}
