% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_dropoutRate1}
\alias{param_dropoutRate1}
\title{Parameter Description: Dropout Rate (1)}
\arguments{
\item{dropoutRate1}{The assumed drop-out rate in the treatment group, default is \code{0}.}
}
\description{
Parameter Description: Dropout Rate (1)
}
\keyword{internal}
