% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_stDevH1}
\alias{param_stDevH1}
\title{Parameter Description: Standard Deviation Under Alternative}
\arguments{
\item{stDevH1}{If specified, the value of the standard deviation under which
the conditional power or sample size recalculation calculation is performed,
default is the value of \code{stDev}.}
}
\description{
Parameter Description: Standard Deviation Under Alternative
}
\keyword{internal}
