% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_palette}
\alias{param_palette}
\title{Parameter Description: Palette}
\arguments{
\item{palette}{The palette, default is \code{"Set1"}.}
}
\description{
Parameter Description: Palette
}
\keyword{internal}
