% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class_core_parameter_set.R
\name{ParameterSet}
\alias{ParameterSet}
\title{Parameter Set}
\description{
Basic class for parameter sets.
}
\details{
The parameter set implements basic functions for a set of parameters.
}
\keyword{internal}
