% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class_core_parameter_set.R
\name{as.data.frame.ParameterSet}
\alias{as.data.frame.ParameterSet}
\title{Coerce Parameter Set to a Data Frame}
\usage{
\method{as.data.frame}{ParameterSet}(
  x,
  row.names = NULL,
  optional = FALSE,
  ...,
  niceColumnNamesEnabled = FALSE,
  includeAllParameters = FALSE
)
}
\arguments{
\item{x}{A \code{\link{FieldSet}} object.}

\item{...}{Ensures that all arguments (starting from the "...") are to be named and
that a warning will be displayed if unknown arguments are passed.}

\item{niceColumnNamesEnabled}{Logical. If \code{TRUE}, nice looking column
names will be used; syntactic names (variable names) otherwise
(see \code{\link[base]{make.names}}).}

\item{includeAllParameters}{Logical. If \code{TRUE}, all available
parameters will be included in the data frame;
a meaningful parameter selection otherwise, default is \code{FALSE}.}
}
\value{
Returns a \code{\link[base]{data.frame}}.
}
\description{
Returns the \code{ParameterSet} as data frame.
}
\details{
Coerces the parameter set to a data frame.
}
\keyword{internal}
