% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_overdispersion_counts}
\alias{param_overdispersion_counts}
\title{Parameter Description: overdispersion for Counts}
\arguments{
\item{overdispersion}{A numeric value that represents the assumed overdispersion of the negative binomial distribution,
default is \code{0}.}
}
\description{
Parameter Description: overdispersion for Counts
}
\keyword{internal}
