% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_populations}
\alias{param_populations}
\title{Parameter Description: Populations}
\arguments{
\item{populations}{The number of populations in a two-sample comparison, default is \code{3}.}
}
\description{
Parameter Description: Populations
}
\keyword{internal}
