% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_effectList}
\alias{param_effectList}
\title{Parameter Description: Effect List}
\arguments{
\item{effectList}{List of subsets, prevalences, and effect sizes with columns and number of rows
reflecting the different situations to consider (see examples).}
}
\description{
Parameter Description: Effect List
}
\keyword{internal}
