% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_pi1_rates}
\alias{param_pi1_rates}
\title{Parameter Description: Pi (1) for Rates}
\arguments{
\item{pi1}{A numeric value or vector that represents the assumed probability in
the active treatment group if two treatment groups
are considered, or the alternative probability for a one treatment group design,
default is \code{seq(0.2, 0.5, 0.1)} (power calculations and simulations) or
\code{seq(0.4, 0.6, 0.1)} (sample size calculations).}
}
\description{
Parameter Description: Pi (1) for Rates
}
\keyword{internal}
