% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class_core_plot_settings.R
\name{PlotSettings}
\alias{PlotSettings}
\title{Plot Settings}
\description{
Class for plot settings.
}
\details{
Collects typical plot settings in an object.
}
\section{Fields}{

\describe{
\item{\code{lineSize}}{The line size.}

\item{\code{pointSize}}{The point size.}

\item{\code{pointColor}}{The point color, e.g., "red" or "blue".}

\item{\code{mainTitleFontSize}}{The main tile font size.}

\item{\code{axesTextFontSize}}{The text font size.}

\item{\code{legendFontSize}}{The legend font size.}

\item{\code{scalingFactor}}{The scaling factor.}
}}

\keyword{internal}
