% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class_core_plot_settings.R
\name{getPlotSettings}
\alias{getPlotSettings}
\title{Get Plot Settings}
\usage{
getPlotSettings(
  lineSize = 0.8,
  pointSize = 3,
  pointColor = NA_character_,
  mainTitleFontSize = 14,
  axesTextFontSize = 10,
  legendFontSize = 11,
  scalingFactor = 1
)
}
\arguments{
\item{lineSize}{The line size, default is \code{0.8}.}

\item{pointSize}{The point size, default is \code{3}.}

\item{pointColor}{The point color (character), default is \code{NA_character_}.}

\item{mainTitleFontSize}{The main title font size, default is \code{14}.}

\item{axesTextFontSize}{The axes text font size, default is \code{10}.}

\item{legendFontSize}{The legend font size, default is \code{11}.}

\item{scalingFactor}{The scaling factor, default is \code{1}.}
}
\description{
Returns a plot settings object.
}
\details{
Returns an object of class \code{PlotSettings} that collects typical plot settings.
}
\keyword{internal}
