% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_thetaH1}
\alias{param_thetaH1}
\title{Parameter Description: Effect Under Alternative}
\arguments{
\item{thetaH1}{If specified, the value of the alternative under which
the conditional power or sample size recalculation calculation is performed. Must be a numeric of length 1.}
}
\description{
Parameter Description: Effect Under Alternative
}
\keyword{internal}
