% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_stDev}
\alias{param_stDev}
\title{Parameter Description: Standard Deviation}
\arguments{
\item{stDev}{The standard deviation under which the sample size or power
calculation is performed, default is \code{1}.
For two-armed trials, it is allowed to specify the standard deviations separately,
i.e., as vector with two elements.
If \code{meanRatio = TRUE} is specified, \code{stDev} defines
the coefficient of variation \code{sigma / mu2}.}
}
\description{
Parameter Description: Standard Deviation
}
\keyword{internal}
