% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_plotPointsEnabled}
\alias{param_plotPointsEnabled}
\title{Parameter Description: Plot Points Enabled}
\arguments{
\item{plotPointsEnabled}{Logical. If \code{TRUE}, additional points will be plotted.}
}
\description{
Parameter Description: Plot Points Enabled
}
\keyword{internal}
