% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_plannedCalendarTime}
\alias{param_plannedCalendarTime}
\title{Parameter Description: Planned Calendar Time}
\arguments{
\item{plannedCalendarTime}{For simulating count data, the time points where an analysis is planned to be performed.
Should be a vector of length \code{kMax}}
}
\description{
Parameter Description: Planned Calendar Time
}
\keyword{internal}
