% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_eventTime}
\alias{param_eventTime}
\title{Parameter Description: Event Time}
\arguments{
\item{eventTime}{The assumed time under which the event rates are calculated, default is \code{12}.}
}
\description{
Parameter Description: Event Time
}
\keyword{internal}
