% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_accrualIntensity}
\alias{param_accrualIntensity}
\title{Parameter Description: Accrual Intensity}
\arguments{
\item{accrualIntensity}{A numeric vector of accrual intensities, default is the relative
intensity \code{0.1} (for details see \code{\link[=getAccrualTime]{getAccrualTime()}}).}
}
\description{
Parameter Description: Accrual Intensity
}
\keyword{internal}
