% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_seed}
\alias{param_seed}
\title{Parameter Description: Seed}
\arguments{
\item{seed}{The seed to reproduce the simulation, default is a random seed.}
}
\description{
Parameter Description: Seed
}
\keyword{internal}
