% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_normalApproximation}
\alias{param_normalApproximation}
\title{Parameter Description: Normal Approximation}
\arguments{
\item{normalApproximation}{The type of computation of the p-values. Default is \code{FALSE} for
testing means (i.e., the t test is used) and \code{TRUE} for testing rates and the hazard ratio.
For testing rates, if \code{normalApproximation = FALSE} is specified, the binomial test
(one sample) or the exact test of Fisher (two samples) is used for calculating the p-values.
In the survival setting \code{normalApproximation = FALSE} has no effect.}
}
\description{
Parameter Description: Normal Approximation
}
\keyword{internal}
