% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{dataSurvival}
\alias{dataSurvival}
\title{One-Arm Dataset of Survival Data}
\format{
A \code{\link[base]{data.frame}} object.
}
\usage{
dataSurvival
}
\description{
A dataset containing the log-rank statistics, events, and allocation ratios of one group.
Use \code{getDataset(dataSurvival)} to create a dataset object that can be processed by \code{\link[=getAnalysisResults]{getAnalysisResults()}}.
}
\keyword{internal}
