% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/f_quality_assurance.R
\name{getSystemIdentifier}
\alias{getSystemIdentifier}
\title{Get System Identifier}
\usage{
getSystemIdentifier(date = NULL)
}
\arguments{
\item{date}{A character string or \code{Date} representing the date. Default is \code{Sys.Date()}.}
}
\value{
A character string representing the unique system identifier.
}
\description{
This function generates a unique system identifier based on the platform, R version, and rpact package version.
}
\examples{
\dontrun{
getSystemIdentifier()
}

}
\keyword{internal}
