% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class_analysis_dataset.R
\name{readDatasets}
\alias{readDatasets}
\title{Read Multiple Datasets}
\usage{
readDatasets(
  file,
  ...,
  header = TRUE,
  sep = ",",
  quote = "\\"",
  dec = ".",
  fill = TRUE,
  comment.char = "",
  fileEncoding = "UTF-8"
)
}
\arguments{
\item{file}{A CSV file (see \code{\link[utils]{read.table}}).}

\item{...}{Further arguments to be passed to \code{\link[utils]{read.table}}.}

\item{header}{A logical value indicating whether the file contains the names of
the variables as its first line.}

\item{sep}{The field separator character. Values on each line of the file are separated
by this character. If sep = "," (the default for \code{readDatasets}) the separator is a comma.}

\item{quote}{The set of quoting characters. To disable quoting altogether, use
quote = "". See scan for the behavior on quotes embedded in quotes. Quoting is only
considered for columns read as character, which is all of them unless \code{colClasses} is specified.}

\item{dec}{The character used in the file for decimal points.}

\item{fill}{logical. If \code{TRUE} then in case the rows have unequal length, blank fields
are implicitly added.}

\item{comment.char}{character: a character vector of length one containing a single character
or an empty string. Use "" to turn off the interpretation of comments altogether.}

\item{fileEncoding}{character string: if non-empty declares the encoding used on a file
(not a connection) so the character data can be re-encoded.
See the 'Encoding' section of the help for file, the 'R Data Import/Export Manual' and 'Note'.}
}
\value{
Returns a \code{\link[base]{list}} of \code{\link{Dataset}} objects.
}
\description{
Reads a data file and returns it as a list of dataset objects.
}
\details{
Reads a file that was written by \code{\link[=writeDatasets]{writeDatasets()}} before.
}
\examples{
\dontrun{
dataFile <- system.file("extdata", "datasets_rates.csv", package = "rpact")
if (dataFile != "") {
    datasets <- readDatasets(dataFile)
    datasets
}
}
}
\seealso{
\itemize{
\item \code{\link[=readDataset]{readDataset()}} for reading a single dataset,
\item \code{\link[=writeDatasets]{writeDatasets()}} for writing multiple datasets,
\item \code{\link[=writeDataset]{writeDataset()}} for writing a single dataset.
}
}
