% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_maxNumberOfSubjects_survival}
\alias{param_maxNumberOfSubjects_survival}
\title{Parameter Description: Maximum Number Of Subjects For Survival Endpoint}
\arguments{
\item{maxNumberOfSubjects}{\code{maxNumberOfSubjects > 0} needs to be specified.
If accrual time and accrual intensity are specified, this will be calculated. Must be a positive integer of length 1.}
}
\description{
Parameter Description: Maximum Number Of Subjects For Survival Endpoint
}
\keyword{internal}
