% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class_simulation_results.R
\name{names.SimulationResults}
\alias{names.SimulationResults}
\title{Names of a Simulation Results Object}
\usage{
\method{names}{SimulationResults}(x)
}
\arguments{
\item{x}{A \code{\link{SimulationResults}} object created by
\code{getSimulationResults[MultiArm/Enrichment][Means/Rates/Survival]}.}
}
\value{
Returns a \code{\link[base]{character}} vector containing the names of the \code{\link{AnalysisResults}} object.
}
\description{
Function to get the names of a \code{\link{SimulationResults}} object.
}
\details{
Returns the names of a simulation results that can be accessed by the user.
}
\keyword{internal}
