% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_tolerance}
\alias{param_tolerance}
\title{Parameter Description: Tolerance}
\arguments{
\item{tolerance}{The numerical tolerance, default is \code{1e-06}. Must be a positive numeric of length 1.}
}
\description{
Parameter Description: Tolerance
}
\keyword{internal}
