% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{dataEnrichmentMeansStratified}
\alias{dataEnrichmentMeansStratified}
\title{Stratified Enrichment Dataset of Means}
\format{
A \code{\link[base]{data.frame}} object.
}
\usage{
dataEnrichmentMeansStratified
}
\description{
A dataset containing the sample sizes, means, and standard deviations of two groups.
Use \code{getDataset(dataEnrichmentMeansStratified)} to create a dataset object that can be processed by \code{\link[=getAnalysisResults]{getAnalysisResults()}}.
}
\keyword{internal}
