% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/f_core_utilities.R
\name{saveOptions}
\alias{saveOptions}
\title{Save Options}
\usage{
saveOptions()
}
\value{
Returns \code{TRUE} if the options were successfully saved, \code{FALSE} otherwise.
}
\description{
Saves the current \code{rpact} options to a configuration file.
}
\details{
This function attempts to save the current \code{rpact} options to a configuration file
located in the user's configuration directory. If the \code{rappdirs} package is not installed,
the function will not perform any action. The options are saved in a YAML file.
}
\examples{
\dontrun{
saveOptions()
}

}
\keyword{internal}
