% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/f_logger.R
\name{getLogLevel}
\alias{getLogLevel}
\title{Get Log Level}
\usage{
getLogLevel()
}
\value{
Returns a \code{\link[base]{character}} of length 1 specifying the current log level.
}
\description{
Returns the current \code{rpact} log level.
}
\details{
This function gets the log level of the \code{rpact} internal log message system.
}
\examples{
# show current log level
getLogLevel()

}
\seealso{
\itemize{
\item \code{\link[=setLogLevel]{setLogLevel()}} for setting the log level,
\item \code{\link[=resetLogLevel]{resetLogLevel()}} for resetting the log level to default.
}
}
\keyword{internal}
