% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_effectMeasure}
\alias{param_effectMeasure}
\title{Parameter Description: Effect Measure}
\arguments{
\item{effectMeasure}{Criterion for treatment arm/population selection, either based on test statistic
(\code{"testStatistic"}) or effect estimate (difference for means and rates or ratio for survival) (\code{"effectEstimate"}),
default is \code{"effectEstimate"}.}
}
\description{
Parameter Description: Effect Measure
}
\keyword{internal}
