% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class_design_plan.R
\name{TrialDesignPlan}
\alias{TrialDesignPlan}
\title{Basic Trial Design Plan}
\description{
Basic class for trial design plans.
}
\details{
\code{TrialDesignPlan} is the basic class for
\itemize{
\item \code{\link{TrialDesignPlanMeans}},
\item \code{\link{TrialDesignPlanRates}}, and
\item \code{\link{TrialDesignPlanSurvival}}.
}
}
\keyword{internal}
