% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_maxInformation}
\alias{param_maxInformation}
\title{Parameter Description: Maximum Information}
\arguments{
\item{maxInformation}{Positive value specifying the maximum information.}
}
\description{
Parameter Description: Maximum Information
}
\keyword{internal}
