% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_allocationRatioPlanned_sampleSize}
\alias{param_allocationRatioPlanned_sampleSize}
\title{Parameter Description: Allocation Ratio Planned With Optimum Option}
\arguments{
\item{allocationRatioPlanned}{The planned allocation ratio \code{n1 / n2} for a two treatment groups
design, default is \code{1}. If \code{allocationRatioPlanned = 0} is entered,
the optimal allocation ratio yielding the smallest overall sample size is determined.}
}
\description{
Parameter Description: Allocation Ratio Planned With Optimum Option
}
\keyword{internal}
