% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class_analysis_stage_results.R
\name{names.StageResults}
\alias{names.StageResults}
\title{Names of a Stage Results Object}
\usage{
\method{names}{StageResults}(x)
}
\arguments{
\item{x}{A \code{\link{StageResults}} object.}
}
\value{
Returns a \code{\link[base]{character}} vector containing the names of the \code{\link{AnalysisResults}} object.
}
\description{
Function to get the names of a \code{\link{StageResults}} object.
}
\details{
Returns the names of stage results that can be accessed by the user.
}
\keyword{internal}
