% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_activeArms}
\alias{param_activeArms}
\title{Parameter Description: Active Arms}
\arguments{
\item{activeArms}{The number of active treatment arms to be compared with control, default is \code{3}.}
}
\description{
Parameter Description: Active Arms
}
\keyword{internal}
