% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_lambda1_counts}
\alias{param_lambda1_counts}
\title{Parameter Description: lambda (1) for Counts}
\arguments{
\item{lambda1}{A numeric value or vector that represents the assumed rate of a homogeneous Poisson process in
the active treatment group, there is no default.}
}
\description{
Parameter Description: lambda (1) for Counts
}
\keyword{internal}
