% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/f_design_futility_bounds.R
\name{summary.FutilityBounds}
\alias{summary.FutilityBounds}
\title{Summarize Futility Bounds}
\usage{
\method{summary}{FutilityBounds}(object, ...)
}
\arguments{
\item{object}{An object of class \code{FutilityBounds}.}

\item{...}{Additional arguments (currently not used).}
}
\description{
S3 summary method for objects of class \code{FutilityBounds}.
}
\details{
Prints a categorized summary of futility bound parameters,
including user-defined, derived, default, and generated values.
}
\examples{
\dontrun{
futilityBounds <- getFutilityBounds(
    design = getDesignInverseNormal(kMax = 2),
    sourceValue = 0.5,
    sourceScale = "condPowerAtObserved",
    targetScale = "zValue"
)
summary(futilityBounds)
}

}
\keyword{internal}
