% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_maxNumberOfIterations}
\alias{param_maxNumberOfIterations}
\title{Parameter Description: Maximum Number Of Iterations}
\arguments{
\item{maxNumberOfIterations}{The number of simulation iterations, default is \code{1000}. Must be a positive integer of length 1.}
}
\description{
Parameter Description: Maximum Number Of Iterations
}
\keyword{internal}
