% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_bindingFutility}
\alias{param_bindingFutility}
\title{Parameter Description: Binding Futility}
\arguments{
\item{bindingFutility}{Logical. If \code{bindingFutility = TRUE} is specified the calculation of
the critical values is affected by the futility bounds and the futility threshold is binding in the
sense that the study must be stopped if the futility condition was reached (default is \code{FALSE}).}
}
\description{
Parameter Description: Binding Futility
}
\keyword{internal}
