% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/f_quality_assurance.R
\name{enableStartupMessages}
\alias{enableStartupMessages}
\title{Enable Startup Messages}
\usage{
enableStartupMessages()
}
\value{
This function does not return a value. It is called for its side effects.
}
\description{
This function enables the startup messages for the \code{rpact} package by setting
the \code{rpact.startup.message.enabled} option to \code{TRUE}.
}
\details{
Once this function is called, the startup messages will remain enabled until
explicitly disabled using the \code{disableStartupMessages()} function. The current
state is saved using the \code{saveOptions()} function.
}
\examples{
\dontrun{
enableStartupMessages()
}

}
\keyword{internal}
