% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_typeOfSelection}
\alias{param_typeOfSelection}
\title{Parameter Description: Type of Selection}
\arguments{
\item{typeOfSelection}{The way the treatment arms or populations are selected at interim.
Five options are available: \code{"best"}, \code{"rbest"}, \code{"epsilon"}, \code{"all"}, and \code{"userDefined"},
default is \code{"best"}.\cr
For \code{"rbest"} (select the \code{rValue} best treatment arms/populations), the parameter \code{rValue} has to be specified,
for \code{"epsilon"} (select treatment arm/population not worse than epsilon compared to the best), the parameter
\code{epsilonValue} has to be specified.
If \code{"userDefined"} is selected, \code{"selectArmsFunction"} or \code{"selectPopulationsFunction"} has to be specified.}
}
\description{
Parameter Description: Type of Selection
}
\keyword{internal}
