% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_gED50}
\alias{param_gED50}
\title{Parameter Description: G ED50}
\arguments{
\item{gED50}{If \code{typeOfShape = "sigmoidEmax"} is selected, \code{gED50} has to be entered
to specify the ED50 of the sigmoid Emax model.}
}
\description{
Parameter Description: G ED50
}
\keyword{internal}
