% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_niceColumnNamesEnabled}
\alias{param_niceColumnNamesEnabled}
\title{Parameter Description: Nice Column Names Enabled}
\arguments{
\item{niceColumnNamesEnabled}{Logical. If \code{TRUE}, nice looking column
names will be used; syntactic names (variable names) otherwise
(see \code{\link[base]{make.names}}).}
}
\description{
Parameter Description: Nice Column Names Enabled
}
\keyword{internal}
