% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_stratifiedAnalysis}
\alias{param_stratifiedAnalysis}
\title{Parameter Description: Stratified Analysis}
\arguments{
\item{stratifiedAnalysis}{Logical. For enrichment designs, typically a stratified analysis should be chosen.
For testing rates, also a non-stratified analysis based on overall data can be performed.
For survival data, only a stratified analysis is possible (see Brannath et al., 2009),
default is \code{TRUE}.}
}
\description{
Parameter Description: Stratified Analysis
}
\keyword{internal}
