% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_maxNumberOfEventsPerStage}
\alias{param_maxNumberOfEventsPerStage}
\title{Parameter Description: Max Number Of Events Per Stage}
\arguments{
\item{maxNumberOfEventsPerStage}{When performing a data driven sample size recalculation,
the numeric vector \code{maxNumberOfEventsPerStage} with length \code{kMax} determines the maximum number
of events per stage (i.e., not cumulated), the first element is not taken into account.}
}
\description{
Parameter Description: Max Number Of Events Per Stage
}
\keyword{internal}
