% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class_analysis_results.R
\name{AnalysisResultsMultiArm}
\alias{AnalysisResultsMultiArm}
\title{Basic Class for Analysis Results Multi-Arm}
\description{
A basic class for multi-arm analysis results.
}
\details{
\code{AnalysisResultsMultiArm} is the basic class for
\itemize{
\item \code{\link{AnalysisResultsMultiArmFisher}},
\item \code{\link{AnalysisResultsMultiArmInverseNormal}}, and
\item \code{\link{AnalysisResultsConditionalDunnett}}.
}
}
\keyword{internal}
