% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/f_quality_assurance.R
\name{print.InstallationQualificationResult}
\alias{print.InstallationQualificationResult}
\title{Print Installation Qualification Result}
\usage{
\method{print}{InstallationQualificationResult}(x, ...)
}
\arguments{
\item{x}{An object of class \code{InstallationQualificationResult} containing the results of the installation qualification.}

\item{...}{Additional arguments passed to or from other methods.}
}
\value{
This function does not return a value. It is called for its side effects of printing the result.
}
\description{
This function prints the details of an \code{InstallationQualificationResult} object in a user-friendly format.
}
\details{
The function displays the result message, followed by the parameters and their values. It skips parameters with \code{NULL} or \code{NA} values.
}
\examples{
\dontrun{
result <- testPackage()
print(result)
}

}
\keyword{internal}
