% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_plannedEvents}
\alias{param_plannedEvents}
\title{Parameter Description: Planned Events}
\arguments{
\item{plannedEvents}{\code{plannedEvents} is a numeric vector of length \code{kMax} (the number of stages of the design)
that determines the number of cumulated (overall) events in survival designs when the interim stages are planned.
For two treatment arms, it is the number of events for both treatment arms.
For multi-arm designs, \code{plannedEvents} refers to the overall number of events for the selected arms plus control.}
}
\description{
Parameter Description: Planned Events
}
\keyword{internal}
