% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_kMax}
\alias{param_kMax}
\title{Parameter Description: Maximum Number of Stages}
\arguments{
\item{kMax}{The maximum number of stages \code{K}.
Must be a positive integer of length 1 (default value is \code{3}).
The maximum selectable \code{kMax} is \code{20} for group sequential or inverse normal and
\code{6} for Fisher combination test designs.}
}
\description{
Parameter Description: Maximum Number of Stages
}
\keyword{internal}
