% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_slope}
\alias{param_slope}
\title{Parameter Description: Slope}
\arguments{
\item{slope}{If \code{typeOfShape = "sigmoidEmax"} is selected, \code{slope} can be entered
to specify the slope of the sigmoid Emax model, default is 1.}
}
\description{
Parameter Description: Slope
}
\keyword{internal}
