% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_directionUpper}
\alias{param_directionUpper}
\title{Parameter Description: Direction Upper}
\arguments{
\item{directionUpper}{Logical. Specifies the direction of the alternative,
only applicable for one-sided testing; default is \code{TRUE}
which means that larger values of the test statistics yield smaller p-values.}
}
\description{
Parameter Description: Direction Upper
}
\keyword{internal}
