% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_threshold}
\alias{param_threshold}
\title{Parameter Description: Threshold}
\arguments{
\item{threshold}{Selection criterion: treatment arm / population is selected only if \code{effectMeasure}
exceeds \code{threshold}, default is \code{-Inf}.
\code{threshold} can also be a vector of length \code{activeArms} referring to
a separate threshold condition over the treatment arms.}
}
\description{
Parameter Description: Threshold
}
\keyword{internal}
