% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_pi2_survival}
\alias{param_pi2_survival}
\title{Parameter Description: Pi (2) for Survival Data}
\arguments{
\item{pi2}{A numeric value that represents the assumed event rate in the control group, default is \code{0.2}.}
}
\description{
Parameter Description: Pi (2) for Survival Data
}
\keyword{internal}
