% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_plotSettings}
\alias{param_plotSettings}
\title{Parameter Description: Plot Settings}
\arguments{
\item{plotSettings}{An object of class \code{PlotSettings} created by \code{\link[=getPlotSettings]{getPlotSettings()}}.}
}
\description{
Parameter Description: Plot Settings
}
\keyword{internal}
