% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class_core_parameter_set.R
\name{kableParameterSet}
\alias{kableParameterSet}
\alias{kable.ParameterSet}
\alias{kable.FieldSet}
\alias{kable.data.frame}
\alias{kable.table}
\alias{kable.matrix}
\alias{kable.array}
\alias{kable.numeric}
\alias{kable.character}
\alias{kable.logical}
\alias{kable}
\title{Create output in Markdown}
\usage{
\method{kable}{ParameterSet}(x, ...)

\method{kable}{FieldSet}(x, ..., enforceRowNames = TRUE, niceColumnNamesEnabled = TRUE)

\method{kable}{data.frame}(x, ...)

\method{kable}{table}(x, ...)

\method{kable}{matrix}(x, ...)

\method{kable}{array}(x, ...)

\method{kable}{numeric}(x, ...)

\method{kable}{character}(x, ...)

\method{kable}{logical}(x, ...)

kable(x, ...)
}
\arguments{
\item{x}{A \code{ParameterSet}. If x does not inherit from class \code{\link{ParameterSet}},
\code{knitr::kable(x)} will be returned.}

\item{...}{Other arguments (see \code{\link[knitr]{kable}}).}
}
\description{
The \code{kable()} function returns the output of the specified object formatted in Markdown.
}
\details{
This function is deprecated and should no longer be used.
Manual use of kable() for rpact result objects is no longer needed,
as the formatting and display will be handled automatically by the rpact package.
Please remove any manual kable() calls from your code to avoid redundancy and potential issues.
The results will be displayed in a consistent format automatically.
}
\keyword{internal}
