% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_successCriterion}
\alias{param_successCriterion}
\title{Parameter Description: Success Criterion}
\arguments{
\item{successCriterion}{Defines when the study is stopped for efficacy at interim.
Two options are available: \code{"all"} stops the trial
if the efficacy criterion is fulfilled for all selected treatment arms/populations,
\code{"atLeastOne"} stops if at least one of the selected treatment arms/populations is shown to be
superior to control at interim, default is \code{"all"}.}
}
\description{
Parameter Description: Success Criterion
}
\keyword{internal}
