% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_three_dots}
\alias{param_three_dots}
\title{Parameter Description: "..."}
\arguments{
\item{...}{Ensures that all arguments (starting from the "...") are to be named and
that a warning will be displayed if unknown arguments are passed.}
}
\description{
Parameter Description: "..."
}
\keyword{internal}
