% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{rawDataTwoArmNormal}
\alias{rawDataTwoArmNormal}
\title{Raw Dataset Of A Two Arm Continuous Outcome With Covariates}
\format{
A \code{\link[base]{data.frame}} object.
}
\usage{
rawDataTwoArmNormal
}
\description{
An artificial dataset that was randomly generated
with simulated normal data. The data set has six variables:
\enumerate{
\item Subject id
\item Stage number
\item Group name
\item An example outcome in that we are interested in
\item The first covariate \emph{gender}
\item The second covariate \emph{covariate}
}
}
\details{
See the vignette "Two-arm analysis for continuous data with covariates from raw data"
to learn how to
\itemize{
\item import raw data from a csv file,
\item calculate estimated adjusted (marginal) means (EMMs, least-squares means) for a linear model, and
\item perform two-arm interim analyses with these data.
}

You can use \code{rawDataTwoArmNormal} to reproduce the examples in the vignette.
}
\keyword{internal}
