% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_plannedSubjects}
\alias{param_plannedSubjects}
\title{Parameter Description: Planned Subjects}
\arguments{
\item{plannedSubjects}{\code{plannedSubjects} is a numeric vector of length \code{kMax} (the number of stages of the design)
that determines the number of cumulated (overall) subjects when the interim stages are planned.
For two treatment arms, it is the number of subjects for both treatment arms.
For multi-arm designs, \code{plannedSubjects} refers to the number of subjects per selected active arm.}
}
\description{
Parameter Description: Planned Subjects
}
\keyword{internal}
