% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_conditionalPower}
\alias{param_conditionalPower}
\title{Parameter Description: Conditional Power}
\arguments{
\item{conditionalPower}{The conditional power for the subsequent stage
under which the sample size recalculation is performed. Must be a positive numeric of length 1.}
}
\description{
Parameter Description: Conditional Power
}
\keyword{internal}
