% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class_performance_score.R
\name{PerformanceScore}
\alias{PerformanceScore}
\title{Performance Score}
\description{
Contains the conditional performance score, its sub-scores and components according to
Herrmann et al. (2020) for a given simulation result.
}
\details{
Use \link{getPerformanceScore} to calculate the performance score.
}
\keyword{internal}
