% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_typeOfShapeRates}
\alias{param_typeOfShapeRates}
\title{Parameter Description: Type Of Shape}
\arguments{
\item{typeOfShape}{The shape of the dose-response relationship over the treatment groups.
This can be either \code{"linear"}, \code{"sigmoidEmax"}, or \code{"userDefined"},
default is \code{"linear"}.\cr
For \code{"linear"}, \code{piMaxVector} specifies the range
of effect sizes for the treatment group with highest response.
If \code{"sigmoidEmax"} is selected, \code{gED50} and \code{slope} has to be entered
to specify the ED50 and the slope of the sigmoid Emax model.
For \code{"sigmoidEmax"}, \code{piMaxVector} specifies the range
of effect sizes for the treatment group with response according to infinite dose.
If \code{"userDefined"} is selected, \code{effectMatrix} has to be entered.}
}
\description{
Parameter Description: Type Of Shape
}
\keyword{internal}
