% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_stageResults}
\alias{param_stageResults}
\title{Parameter Description: Stage Results}
\arguments{
\item{stageResults}{The results at given stage, obtained from \code{\link[=getStageResults]{getStageResults()}}.}
}
\description{
Parameter Description: Stage Results
}
\keyword{internal}
