% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_userAlphaSpending}
\alias{param_userAlphaSpending}
\title{Parameter Description: User Alpha Spending}
\arguments{
\item{userAlphaSpending}{The user defined alpha spending.
Numeric vector of length \code{kMax} containing the cumulative
alpha-spending (Type I error rate) up to each interim stage: \code{0 <= alpha_1 <= ... <= alpha_K <= alpha}.}
}
\description{
Parameter Description: User Alpha Spending
}
\keyword{internal}
