% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_hazardRatio}
\alias{param_hazardRatio}
\title{Parameter Description: Hazard Ratio}
\arguments{
\item{hazardRatio}{The vector of hazard ratios under consideration.
If the event or hazard rates in both treatment groups are defined, the hazard ratio needs
not to be specified as it is calculated, there is no default. Must be a positive numeric of length 1.}
}
\description{
Parameter Description: Hazard Ratio
}
\keyword{internal}
