% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/f_analysis_utilities.R
\name{getLongFormat}
\alias{getLongFormat}
\title{Get Long Format}
\usage{
getLongFormat(dataInput)
}
\value{
A \code{\link[base]{data.frame}} will be returned.
}
\description{
Returns the specified dataset as a \code{\link[base]{data.frame}} in so-called long format.
}
\details{
In the long format (narrow, stacked), the data are presented with one column containing
all the values and another column listing the context of the value, i.e.,
the data for the different groups are in one column and the dataset contains an additional "group" column.
}
\seealso{
\code{\link[=getWideFormat]{getWideFormat()}} for returning the dataset as a \code{\link[base]{data.frame}} in wide format.
}
\keyword{internal}
