% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class_analysis_results.R
\name{as.data.frame.AnalysisResults}
\alias{as.data.frame.AnalysisResults}
\title{Coerce AnalysisResults to a Data Frame}
\usage{
\method{as.data.frame}{AnalysisResults}(
  x,
  row.names = NULL,
  optional = FALSE,
  ...,
  niceColumnNamesEnabled = FALSE
)
}
\arguments{
\item{x}{An \code{\link{AnalysisResults}} object created by \code{\link[=getAnalysisResults]{getAnalysisResults()}}.}

\item{...}{Ensures that all arguments (starting from the "...") are to be named and
that a warning will be displayed if unknown arguments are passed.}

\item{niceColumnNamesEnabled}{Logical. If \code{TRUE}, nice looking column
names will be used; syntactic names (variable names) otherwise
(see \code{\link[base]{make.names}}).}
}
\value{
Returns a \code{\link[base]{data.frame}}.
}
\description{
Returns the \code{\link{AnalysisResults}} object as data frame.
}
\details{
Coerces the analysis results to a data frame.
}
\keyword{internal}
