% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_calcEventsFunction}
\alias{param_calcEventsFunction}
\title{Parameter Description: Calculate Events Function}
\arguments{
\item{calcEventsFunction}{Optionally, a function can be entered that defines the way of performing the sample size
recalculation. By default, event number recalculation is performed with conditional power and specified
\code{minNumberOfEventsPerStage} and \code{maxNumberOfEventsPerStage} (see details and examples).}
}
\description{
Parameter Description: Calculate Events Function
}
\keyword{internal}
