% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class_analysis_results.R
\name{AnalysisResultsMultiHypotheses}
\alias{AnalysisResultsMultiHypotheses}
\title{Basic Class for Analysis Results Multi-Hypotheses}
\description{
A basic class for multi-hypotheses analysis results.
}
\details{
\code{AnalysisResultsMultiHypotheses} is the basic class for
\itemize{
\item \code{\link{AnalysisResultsMultiArm}} and
\item \code{\link{AnalysisResultsEnrichment}}.
}
}
\keyword{internal}
