% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class_analysis_dataset.R
\name{print.Dataset}
\alias{print.Dataset}
\title{Print Dataset Values}
\usage{
\method{print}{Dataset}(
  x,
  ...,
  markdown = NA,
  output = c("list", "long", "wide", "r", "rComplete")
)
}
\arguments{
\item{x}{A \code{\link{Dataset}} object.}

\item{...}{Ensures that all arguments (starting from the "...") are to be named and
that a warning will be displayed if unknown arguments are passed.}

\item{markdown}{If \code{TRUE}, the output will be created in Markdown.}

\item{output}{A character defining the output type, default is "list".}
}
\description{
\code{print} prints its \code{\link{Dataset}} argument and returns it invisibly (via \code{invisible(x)}).
}
\details{
Prints the dataset.
}
\keyword{internal}
