% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_digits}
\alias{param_digits}
\title{Parameter Description: Digits}
\arguments{
\item{digits}{Defines how many digits are to be used for numeric values. Must be a positive integer of length 1.}
}
\description{
Parameter Description: Digits
}
\keyword{internal}
