% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/f_quality_assurance.R
\name{test_plan_section}
\alias{test_plan_section}
\title{Test Plan Section}
\usage{
test_plan_section(section)
}
\arguments{
\item{section}{The section title or description.}
}
\description{
The section title or description will be used in the formal validation documentation.
For more information visit \url{https://www.rpact.com}
}
\keyword{internal}
