% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_maxNumberOfSubjectsPerStage}
\alias{param_maxNumberOfSubjectsPerStage}
\title{Parameter Description: Maximum Number Of Subjects Per Stage}
\arguments{
\item{maxNumberOfSubjectsPerStage}{When performing a data driven sample size recalculation,
the numeric vector \code{maxNumberOfSubjectsPerStage} with length \code{kMax} determines the maximum number
of subjects per stage (i.e., not cumulated), the first element is not taken into account.
For two treatment arms, it is the number of subjects for both treatment arms.
For multi-arm designs \code{maxNumberOfSubjectsPerStage} refers
to the maximum number of subjects per selected active arm.}
}
\description{
Parameter Description: Maximum Number Of Subjects Per Stage
}
\keyword{internal}
