% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/f_core_utilities.R
\name{printCitation}
\alias{printCitation}
\title{Print Citation}
\usage{
printCitation(inclusiveR = TRUE, language = "en", markdown = NA)
}
\arguments{
\item{inclusiveR}{If \code{TRUE} (default) the information on how to cite the base R system in publications will be added.}

\item{language}{Language code to use for the output, default is "en".}

\item{markdown}{If \code{TRUE}, the output will be created in Markdown.}
}
\description{
How to cite \code{rpact} and \code{R} in publications.
}
\details{
This function shows how to cite \code{rpact} and \code{R} (\code{inclusiveR = TRUE}) in publications.
}
\examples{
printCitation()

}
\keyword{internal}
