% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/f_core_utilities.R
\name{getParameterName}
\alias{getParameterName}
\title{Get Parameter Name}
\usage{
getParameterName(obj, parameterCaption)
}
\arguments{
\item{obj}{The rpact result object.}

\item{parameterCaption}{The parameter caption.}
}
\value{
Returns a \code{\link[base]{character}} of specifying the corresponding name of a given parameter caption.
Returns \code{NULL} if the specified \code{parameterCaption} does not exist.
}
\description{
Returns the parameter name for a given object and parameter caption.
}
\details{
This function identifies and returns the parameter name for a given caption
that will be used in print outputs of an rpact result object.
}
\examples{
\dontrun{
getParameterName(getDesignInverseNormal(), "Maximum number of stages")
}

}
\seealso{
\code{\link[=getParameterCaption]{getParameterCaption()}} for getting the parameter caption for a given name.
}
\keyword{internal}
