% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_alternative}
\alias{param_alternative}
\title{Parameter Description: Alternative}
\arguments{
\item{alternative}{The alternative hypothesis value for testing means. This can be a vector of assumed
alternatives, default is \code{seq(0, 1, 0.2)} (power calculations) or \code{seq(0.2, 1, 0.2)} (sample size calculations).}
}
\description{
Parameter Description: Alternative
}
\keyword{internal}
