% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_theta}
\alias{param_theta}
\title{Parameter Description: Theta}
\arguments{
\item{theta}{A vector of standardized effect sizes (theta values), default is a sequence from -1 to 1.}
}
\description{
Parameter Description: Theta
}
\keyword{internal}
