% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class_analysis_results.R
\name{AnalysisResultsEnrichment}
\alias{AnalysisResultsEnrichment}
\title{Basic Class for Analysis Results Enrichment}
\description{
A basic class for enrichment analysis results.
}
\details{
\code{AnalysisResultsEnrichment} is the basic class for
\itemize{
\item \code{\link{AnalysisResultsEnrichmentFisher}} and
\item \code{\link{AnalysisResultsEnrichmentInverseNormal}}.
}
}
\keyword{internal}
