% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_accrualIntensity_counts}
\alias{param_accrualIntensity_counts}
\title{Parameter Description: accrualIntensity for Counts}
\arguments{
\item{accrualIntensity}{If specified, the assumed accrual intensities for the study, there is no default.}
}
\description{
Parameter Description: accrualIntensity for Counts
}
\keyword{internal}
