% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_rValue}
\alias{param_rValue}
\title{Parameter Description: R Value}
\arguments{
\item{rValue}{For \code{typeOfSelection = "rbest"} (select the \code{rValue} best treatment arms / populations),
the parameter \code{rValue} has to be specified.}
}
\description{
Parameter Description: R Value
}
\keyword{internal}
