% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_informationRates}
\alias{param_informationRates}
\title{Parameter Description: Information Rates}
\arguments{
\item{informationRates}{The information rates t_1, ..., t_kMax (that must be fixed prior to the trial),
default is \code{(1:kMax) / kMax}. For the weighted inverse normal design, the weights are derived
through w_1 = sqrt(t_1), and w_k = sqrt(t_k - t_(k-1)). For the weighted Fisher's combination test, the
weights (scales) are w_k = sqrt((t_k - t_(k-1)) / t_1) (see the documentation).}
}
\description{
Parameter Description: Information Rates
}
\keyword{internal}
