% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_calcSubjectsFunction}
\alias{param_calcSubjectsFunction}
\title{Parameter Description: Calculate Subjects Function}
\arguments{
\item{calcSubjectsFunction}{Optionally, a function can be entered that defines the way of performing the sample size
recalculation. By default, sample size recalculation is performed with conditional power and specified
\code{minNumberOfSubjectsPerStage} and \code{maxNumberOfSubjectsPerStage} (see details and examples).}
}
\description{
Parameter Description: Calculate Subjects Function
}
\keyword{internal}
