% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_nMax}
\alias{param_nMax}
\title{Parameter Description: N_max}
\arguments{
\item{nMax}{The maximum sample size. Must be a positive integer of length 1.}
}
\description{
Parameter Description: N_max
}
\keyword{internal}
