% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class_design_set.R
\name{names.TrialDesignSet}
\alias{names.TrialDesignSet}
\title{Names of a Trial Design Set Object}
\usage{
\method{names}{TrialDesignSet}(x)
}
\arguments{
\item{x}{A \code{\link{TrialDesignSet}} object.}
}
\value{
Returns a \code{\link[base]{character}} vector containing the names of the \code{\link{AnalysisResults}} object.
}
\description{
Function to get the names of a \code{\link{TrialDesignSet}} object.
}
\details{
Returns the names of a design set that can be accessed by the user.
}
\examples{
\dontrun{
designSet <- getDesignSet(design = getDesignGroupSequential(), alpha = c(0.01, 0.05))
names(designSet)
}

}
\keyword{internal}
