% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/f_design_futility_bounds.R
\name{print.FutilityBounds}
\alias{print.FutilityBounds}
\title{Print Futility Bounds}
\usage{
\method{print}{FutilityBounds}(x, ...)
}
\arguments{
\item{x}{An object of class \code{FutilityBounds}.}

\item{...}{Additional arguments passed to \code{print.default}.}
}
\description{
S3 print method for objects of class \code{FutilityBounds}. Prints the futility bounds as a numeric vector.
}
\keyword{internal}
