% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_accrualIntensityType}
\alias{param_accrualIntensityType}
\title{Parameter Description: Accrual Intensity Type}
\arguments{
\item{accrualIntensityType}{A character value specifying the accrual intensity input type.
Must be one of \code{"auto"}, \code{"absolute"}, or \code{"relative"}; default is \code{"auto"},
i.e., if all values are < 1 the type is \code{"relative"}, otherwise it is \code{"absolute"}.}
}
\description{
Parameter Description: Accrual Intensity Type
}
\keyword{internal}
