% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/f_core_utilities.R
\name{resetOptions}
\alias{resetOptions}
\title{Reset Options}
\usage{
resetOptions(persist = TRUE)
}
\arguments{
\item{persist}{A logical value indicating whether the reset options should be saved persistently.
If \code{TRUE}, the options will be saved after resetting. Default is \code{TRUE}.}
}
\value{
Returns \code{TRUE} if the options were successfully reset, \code{FALSE} otherwise.
}
\description{
Resets the \code{rpact} options to their default values.
}
\details{
This function resets all \code{rpact} options to their default values. If the \code{persist} parameter is set to \code{TRUE},
the reset options will be saved to a configuration file.
}
\examples{
\dontrun{
resetOptions()
resetOptions(persist = FALSE)
}

}
\keyword{internal}
