% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_piecewiseSurvivalTime}
\alias{param_piecewiseSurvivalTime}
\title{Parameter Description: Piecewise Survival Time}
\arguments{
\item{piecewiseSurvivalTime}{A vector that specifies the time intervals for the piecewise
definition of the exponential survival time cumulative distribution function \cr
(for details see \code{\link[=getPiecewiseSurvivalTime]{getPiecewiseSurvivalTime()}}).}
}
\description{
Parameter Description: Piecewise Survival Time
}
\keyword{internal}
