% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class_design_set.R
\name{print.TrialDesignSummaries}
\alias{print.TrialDesignSummaries}
\title{Print Trial Design Summaries}
\usage{
\method{print}{TrialDesignSummaries}(x, ...)
}
\arguments{
\item{x}{a \code{TrialDesignSummaries} object to print.}

\item{...}{further arguments passed to or from other methods.}
}
\description{
Generic function to print a \code{TrialDesignSummaries} object.
}
