% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_selectArmsFunction}
\alias{param_selectArmsFunction}
\title{Parameter Description: Select Arms Function}
\arguments{
\item{selectArmsFunction}{Optionally, a function can be entered that defines the way of how treatment arms
are selected. This function is allowed to depend on \code{effectVector} with length \code{activeArms},
\code{stage}, \code{conditionalPower}, \code{conditionalCriticalValue}, \code{plannedSubjects/plannedEvents},
\code{allocationRatioPlanned}, \code{selectedArms}, \code{thetaH1} (for means and survival), \code{stDevH1} (for means),
\code{overallEffects}, and for rates additionally: \code{piTreatmentsH1}, \code{piControlH1}, \code{overallRates}, and
\code{overallRatesControl} (see examples).}
}
\description{
Parameter Description: Select Arms Function
}
\keyword{internal}
