% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/f_analysis_utilities.R
\name{getWideFormat}
\alias{getWideFormat}
\title{Get Wide Format}
\usage{
getWideFormat(dataInput)
}
\value{
A \code{\link[base]{data.frame}} will be returned.
}
\description{
Returns the specified dataset as a \code{\link[base]{data.frame}} in so-called wide format.
}
\details{
In the wide format (unstacked), the data are presented with each different data variable in a separate column, i.e.,
the different groups are in separate columns.
}
\seealso{
\code{\link[=getLongFormat]{getLongFormat()}} for returning the dataset as a \code{\link[base]{data.frame}} in long format.
}
\keyword{internal}
