% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class_design_set.R
\name{TrialDesignSet}
\alias{TrialDesignSet}
\title{Class for trial design sets.}
\description{
\code{TrialDesignSet} is a class for creating a collection of different trial designs.
}
\details{
This object cannot be created directly; better use \code{\link[=getDesignSet]{getDesignSet()}}
with suitable arguments to create a set of designs.
}
\section{Fields}{

\describe{
\item{\code{designs}}{The trial designs to be compared.}

\item{\code{design}}{The trial design.}

\item{\code{variedParameters}}{A character vector containing the names of the parameters that vary between designs.}
}}

\seealso{
\code{\link[=getDesignSet]{getDesignSet()}}
}
\keyword{internal}
