% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_three_dots_plot}
\alias{param_three_dots_plot}
\title{Parameter Description: "..." (optional plot arguments)}
\arguments{
\item{...}{Optional plot arguments. At the moment \code{xlim} and \code{ylim} are implemented
for changing x or y axis limits without dropping data observations.}
}
\description{
Parameter Description: "..." (optional plot arguments)
}
\keyword{internal}
