% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class_analysis_results.R
\name{AnalysisResults}
\alias{AnalysisResults}
\title{Basic Class for Analysis Results}
\description{
A basic class for analysis results.
}
\details{
\code{AnalysisResults} is the basic class for
\itemize{
\item \code{\link{AnalysisResultsFisher}},
\item \code{\link{AnalysisResultsGroupSequential}},
\item \code{\link{AnalysisResultsInverseNormal}},
\item \code{\link{AnalysisResultsMultiArmFisher}},
\item \code{\link{AnalysisResultsMultiArmInverseNormal}},
\item \code{\link{AnalysisResultsConditionalDunnett}},
\item \code{\link{AnalysisResultsEnrichmentFisher}},
\item \code{\link{AnalysisResultsEnrichmentInverseNormal}}.
}
}
\keyword{internal}
