% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_fixedExposureTime_counts}
\alias{param_fixedExposureTime_counts}
\title{Parameter Description: fixedExposureTime for Counts}
\arguments{
\item{fixedExposureTime}{If specified, the fixed time of exposure per subject for count data, there is no default.}
}
\description{
Parameter Description: fixedExposureTime for Counts
}
\keyword{internal}
