% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_followUpTime_counts}
\alias{param_followUpTime_counts}
\title{Parameter Description: followUpTime for Counts}
\arguments{
\item{followUpTime}{If specified, the assumed (additional) follow-up time for the study, there is no default.
The total study duration is \code{accrualTime + followUpTime}.}
}
\description{
Parameter Description: followUpTime for Counts
}
\keyword{internal}
