% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_doseLevels}
\alias{param_doseLevels}
\title{Parameter Description: Dose Levels}
\arguments{
\item{doseLevels}{The dose levels for the dose response relationship.
If not specified, these dose levels are \code{1,...,activeArms}.}
}
\description{
Parameter Description: Dose Levels
}
\keyword{internal}
