% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_maxNumberOfSubjects}
\alias{param_maxNumberOfSubjects}
\title{Parameter Description: Maximum Number Of Subjects}
\arguments{
\item{maxNumberOfSubjects}{\code{maxNumberOfSubjects > 0} needs to be specified for power calculations or calculation
of necessary follow-up (count data). For two treatment arms, it is the maximum number of subjects for both treatment arms.}
}
\description{
Parameter Description: Maximum Number Of Subjects
}
\keyword{internal}
