% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_beta}
\alias{param_beta}
\title{Parameter Description: Beta}
\arguments{
\item{beta}{Type II error rate, necessary for providing sample size calculations
(e.g., \code{\link[=getSampleSizeMeans]{getSampleSizeMeans()}}), beta spending function designs,
or optimum designs, default is \code{0.20}. Must be a positive numeric of length 1.}
}
\description{
Parameter Description: Beta
}
\keyword{internal}
