% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class_summary.R
\name{SummaryFactory}
\alias{SummaryFactory}
\title{Summary Factory}
\description{
Basic class for summaries
}
\keyword{internal}
