% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_theta_counts}
\alias{param_theta_counts}
\title{Parameter Description: theta for Counts}
\arguments{
\item{theta}{A numeric value or vector that represents the assumed mean ratios lambda1/lambda2 of a homogeneous
Poisson process, there is no default.}
}
\description{
Parameter Description: theta for Counts
}
\keyword{internal}
